// ******************
// ASYMMETRIC CYPHER
// ******************

// **************
// GENERATE KEYS
// **************


/*------------------
 Show Password asym.
-------------------*/

(function($) {

  "use strict";

  // Show tooltip only if not on touch device
  // On touch device, the tooltip is deactivated on mobile and tablet.

  var hasNotTouch = true;
  if (Modernizr.touch) { hasNotTouch = false; }

  var element = $('[data-bs-toggle="tooltip"]');

  $("#show_password").hover(function() {
    if ($('#show_password').hasClass('bi-eye-slash-fill')) {
      if (hasNotTouch) {
        $(element).attr('data-original-title', 'Masquer le mot de passe').tooltip('show');
      }
    }
    else {
      if (hasNotTouch) {
        $(element).attr('data-original-title', 'Afficher le mot de passe').tooltip('show');
      }
    }
  });

  $("#show_password").click(function() {

    if ($('#show_password').hasClass('bi-eye-fill')) {
      $('#asym_create_form_password').attr('type', 'text');
      $('#show_password').removeClass('bi-eye-fill').addClass('bi-eye-slash-fill');

      if (hasNotTouch) {
        $(element).attr('data-original-title', 'Masquer le mot de passe').tooltip('show');
      }
    }
    else {
      $('#asym_create_form_password').attr('type', 'password');
      $('#show_password').removeClass('bi-eye-slash-fill').addClass('bi-eye-fill');

      if (hasNotTouch) {
        $(element).attr('data-original-title', 'Afficher le mot de passe').tooltip('show');
      }
    }
  });

})(window.jQuery);



(function() {

    "use strict";
    
    var generateKeys = document.getElementById('generateKeys');

    if(generateKeys){
        generateKeys.addEventListener('click', function(e){
            var asymKeyName = document.getElementById('asymKeyName').value;
            var asymKeyEmail = document.getElementById('asymKeyEmail').value;

            var errorGenerateKeys = document.getElementById("errorGenerateKeys");
            errorGenerateKeys.innerHTML = '';
            
            var asym_create_form_password = document.getElementById('asym_create_form_password').value;
            if('' == asym_create_form_password || asym_create_form_password.length < 16) {                           
                document.getElementById("errorGenerateKeys").innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Indiquez un mot de passe d\'au moins 16 caractères pour protéger votre clé privée.</p>'
                return;
            }

            (async () => {
                const { privateKey, publicKey, revocationCertificate } = await openpgp.generateKey({
                    type: 'ecc', // type of the key, defaults to ECC
                    curve: 'curve25519', // ECC curve name, defaults to curve25519
                    userIDs: [{ name: asymKeyName, email: asymKeyEmail }],
                    passphrase: asym_create_form_password,
                    format: 'armored' // output key format, defaults to 'armored' (other options: 'binary' or 'object')
                });

                const publicKeyCheck = await openpgp.readKey({ armoredKey: publicKey });
                
                // -----BEGIN PGP PRIVATE KEY BLOCK-----
                addText(privateKey, "privateKeyGenerated");
                removeDisabled('privateKeyGenerated');

                // -----BEGIN PGP PUBLIC KEY BLOCK-----
                addText(publicKey, "publicKeyGenerated");
                removeDisabled('publicKeyGenerated');
                
                // -----BEGIN PGP PUBLIC KEY BLOCK-----
                // Comment: This is a revocation certificate
                addText(revocationCertificate, "revocationCertificate");
                removeDisabled('revocationCertificate');

                addText(publicKeyCheck.keyPacket.keyID.toHex().toUpperCase(), "pubKeyID");
                removeDisabled('pubKeyID');

                setDisabled('generateKeys');
            })();
        });
    }

    // cancel
    var cancelGenerateKeys = document.getElementById('cancelGenerateKeys');
    if(cancelGenerateKeys){
        cancelGenerateKeys.addEventListener('click', function(e){
            e.preventDefault();
            var errorGenerateKeys = document.getElementById("errorGenerateKeys");
            errorGenerateKeys.innerHTML = '';

            const privateKeyGenerated = document.getElementById("privateKeyGenerated");
                  privateKeyGenerated.value = '';

            const publicKeyGenerated = document.getElementById("publicKeyGenerated");
                  publicKeyGenerated.value = '';

            const revocationCertificate = document.getElementById("revocationCertificate");
                  revocationCertificate.value = '';

            const pubKeyID = document.getElementById("pubKeyID");
                  pubKeyID.value = '';

            removeDisabled('generateKeys');
            setDisabled('privateKeyGenerated');
            setDisabled('publicKeyGenerated');
            setDisabled('revocationCertificate');
        })
    }
})();