// ************************
// ASYMMETRIC CYPHER (AES)
// ************************

// **************
// GENERATE KEYS
// **************

(function() {

    "use strict";
    
    var btnInfoKeyPub = document.getElementById('btnInfoKeyPub');

    if(btnInfoKeyPub){
        btnInfoKeyPub.addEventListener('click', function(e){
            e.preventDefault();
            
            var errorInfoKeyPub = document.getElementById('errorInfoKeyPub');
            errorInfoKeyPub.innerHTML = '';
            
            var inputInfoKeyPub = document.getElementById('inputInfoKeyPub').value;
            if('' == inputInfoKeyPub) {                           
                document.getElementById("errorInfoKeyPub").innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Indiquez une clé publique.</p>'
                return;
            }

            (async () => {

                function buf2hex(buffer) { // buffer is an ArrayBuffer
                    return [...new Uint8Array(buffer)]
                        .map(x => x.toString(16).padStart(2, '0'))
                        .join('');
                }

                function separatorEveryNChar(str, separator, nChar) {
                    var chain = '';
                    var resultTab = [];
                    for (var i=0; i < str.length; i++) {
                        chain += str[i];

                        if(chain.length % nChar === 0) {
                            var temp = chain.slice(chain.length - nChar);
                            resultTab.push(temp);
                        }
                    }
                    return resultTab.join(separator);
                }
                
                try {
                    const pubKeyCheck = await openpgp.readKey({ armoredKey: inputInfoKeyPub });
                } 
                catch (e) {
                    document.getElementById("errorInfoKeyPub").innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Informations de clé : ' + e.message + '.</p>'
                    return;
                }

                const pubKeyCheck = await openpgp.readKey({ armoredKey: inputInfoKeyPub });
                var expirationDate = pubKeyCheck.keyPacket.expirationTimeV3
                if(pubKeyCheck.keyPacket.expirationTimeV3 == 0){
                    expirationDate = "Jamais";
                }

                var asymKeyName = pubKeyCheck.users[0].userID.name;
                if(asymKeyName == 0){
                    asymKeyName = "Aucun nom";
                }

                var asymKeyEmail = pubKeyCheck.users[0].userID.email;
                if(asymKeyEmail == 0){
                    asymKeyEmail = "Aucun email";
                }

                var fingerprintView = buf2hex(pubKeyCheck.keyPacket.fingerprint);

                fingerprintView = separatorEveryNChar(fingerprintView, '  ', 4);
                var outputInfoKeyPub = document.getElementById('outputInfoKeyPub');

                outputInfoKeyPub.value =  
                    "Propriétaire : " + asymKeyName + ' < ' + asymKeyEmail + ' >'
                    + "\n" + "Créée le : " + pubKeyCheck.keyPacket.created 
                    + "\n" + "Date d'expiration : " + expirationDate
                    + "\n" + "Empreinte : " + fingerprintView.toUpperCase() 
                    + "\n" + "Identifiant : " + pubKeyCheck.keyPacket.keyID.toHex().toUpperCase()
                    + "\n" + "Version : " + pubKeyCheck.keyPacket.version 
                ;
                removeDisabled('outputInfoKeyPub');

                // console.log(pubKeyCheck.users[0]);
                // console.log(pubKeyCheck.users[0].userID);
                // console.log(pubKeyCheck.curve);

            })();
        });

        // cancel
        var cancelInfoKeyPub = document.getElementById('cancelInfoKeyPub');
        if(cancelInfoKeyPub){
            cancelInfoKeyPub.addEventListener('click', function(e){
                e.preventDefault();
                var errorInfoKeyPub = document.getElementById("errorInfoKeyPub");
                errorInfoKeyPub.innerHTML = '';

                const inputInfoKeyPub = document.getElementById("inputInfoKeyPub");
                    inputInfoKeyPub.value = '';

                const outputInfoKeyPub = document.getElementById("outputInfoKeyPub");
                    outputInfoKeyPub.value = '';

                removeDisabled('crypt');
            })
        }
    }
})();