
// *****************
// SYMMETRIC CYPHER
// *****************

(function() {

    "use strict";

    // USEFUL FUNCTIONS
    // ----------------

    // https://cryptojs.gitbook.io/docs/
    function createSymKey(passphrase, salt) {

        var symKey = CryptoJS.PBKDF2(passphrase, salt, {
            keySize: 512 / 32,
            iterations: 100000
        });
        return symKey;
    };

    // / END - USEFUL FUNCTIONS


    // ----------------------
    // 4 TABS
    // ----------------------
    var styleCover = 'position:absolute;width:100%;height:100%;opacity:0.3;z-index:100;background:#fff;';
    
    // ----------------------
    // TAB 1 - KEY GENERATION
    // ----------------------

    /*------------------
    Show Password asym.
    -------------------*/

    (function($) {

    "use strict";

    // Show tooltip only if not on touch device
    // On touch device, the tooltip is deactivated on mobile and tablet.

    var hasNotTouch = true;
    if (Modernizr.touch) { hasNotTouch = false; }

    var element = $('[data-bs-toggle="tooltip"]');

    $("#show_password_sym").hover(function() {
        if ($('#show_password_sym').hasClass('bi-eye-slash-fill')) {
        if (hasNotTouch) {
            $(element).attr('data-original-title', 'Masquer le mot de passe').tooltip('show');
        }
        }
        else {
        if (hasNotTouch) {
            $(element).attr('data-original-title', 'Afficher le mot de passe').tooltip('show');
        }
        }
    });

    $("#show_password_sym").click(function() {

        if ($('#show_password_sym').hasClass('bi-eye-fill')) {
        $('#passwordToCreateKey').attr('type', 'text');
        $('#show_password_sym').removeClass('bi-eye-fill').addClass('bi-eye-slash-fill');

        if (hasNotTouch) {
            $(element).attr('data-original-title', 'Masquer le mot de passe').tooltip('show');
        }
        }
        else {
        $('#passwordToCreateKey').attr('type', 'password');
        $('#show_password_sym').removeClass('bi-eye-slash-fill').addClass('bi-eye-fill');

        if (hasNotTouch) {
            $(element).attr('data-original-title', 'Afficher le mot de passe').tooltip('show');
        }
        }
    });

    })(window.jQuery);




    function keyPromiseSuccess() {
        var elem = document.body;
        elem.style.cssText = '';
        
        var elemSalt = document.getElementById('salt');
        elemSalt.removeAttribute('disabled', '');

        var elemGeneratedKeyLocation = document.getElementById('generatedKeyLocation');
        elemGeneratedKeyLocation.removeAttribute('disabled', '');
    }

    function keyPromiseError() {
        document.getElementById("errorSymGenerateKey").innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Un problème est survenu lors de la génération de clé !</p>'
        return;
    }           

    function keyGeneration() {
        try {           
            var passwordToCreateKey = document.getElementById('passwordToCreateKey').value;
            var saltToCreateKey = document.getElementById('salt').value;
            var symKeyGenerated = createSymKey(passwordToCreateKey, saltToCreateKey).toString().toUpperCase();
            addText(symKeyGenerated, 'generatedKeyLocation');
        }
        catch(error) {
            document.getElementById("errorSymGenerateKey").innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Erreur de génération de clé symétrique : ' + error.message + '.</p>'
            return;
        }
    }

    // button to generate the key
    var buttonGenerateKey = document.getElementById('buttonGenerateKey');
    if(buttonGenerateKey){
        buttonGenerateKey.addEventListener('click', function(e){
            // remove potentiel error displayed
            var errorSymGenerateKey = document.getElementById("errorSymGenerateKey");
            errorSymGenerateKey.innerHTML = '';

            var passwordToCreateKey = document.getElementById('passwordToCreateKey').value;
            var salt = document.getElementById('salt').value;
            if('' == passwordToCreateKey || passwordToCreateKey.length < 40 ) {                           
                document.getElementById("errorSymGenerateKey").innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Merci d\'indiquer un mot de passe ou phrase secrète d\'au moins 40 caractères pour créer votre clé symétrique.</p>'
                return;
            }

            if('' == salt) {                           
                document.getElementById("errorSymGenerateKey").innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Merci d\'indiquer un sel après l\'avoir généré sur l\'onglet "Entropie du sel".</p>'
                return;
            }

            const keyGeneratorPromise = new Promise((resolve, reject) => {
                var elem = document.body;
                elem.style.cssText = styleCover;

                setTimeout(() => {
                    resolve(keyGeneration());
                }, 50);
            });
            keyGeneratorPromise
            .then(keyPromiseSuccess, keyPromiseError);
        });

        // cancel
        var cancelSymGenerateKey = document.getElementById('cancelSymGenerateKey');
        if(cancelSymGenerateKey){
            cancelSymGenerateKey.addEventListener('click', function(e){
                e.preventDefault();
                var errorSymGenerateKey = document.getElementById("errorSymGenerateKey");
                errorSymGenerateKey.innerHTML = '';

                const passwordToCreateKey = document.getElementById("passwordToCreateKey");
                    passwordToCreateKey.value = '';

                const salt = document.getElementById("salt");
                    salt.value = '';

                const generatedKeyLocation = document.getElementById("generatedKeyLocation");
                    generatedKeyLocation.value = '';

                removeDisabled('buttonGenerateKey');
                setDisabled('generatedKeyLocation');
            })
        }
    }


    // ----------------------
    // TAB 2 - ENCRYPT
    // ----------------------

    // button to encrypt
    var buttonToEncrypt = document.getElementById('buttonToEncrypt');
    if(buttonToEncrypt){
        buttonToEncrypt.addEventListener('click', function(e){

            var errorSymEncrypt = document.getElementById("errorSymEncrypt");
            errorSymEncrypt.innerHTML = '';

            var key = document.getElementById('symEncryptKeyLocation').value;
            var messageToEncrypt = document.getElementById('message').value;

            if('' == key) {                           
                document.getElementById("errorSymEncrypt").innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Merci d\'indiquer votre clé symétrique.</p>'
                return;
            }

            if('' == messageToEncrypt) {                           
                document.getElementById("errorSymEncrypt").innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Merci d\'indiquer votre contenu textuel à chiffrer.</p>'
                return;
            }

            try {           
                // Encrypt
                var encrypted = CryptoJS.AES.encrypt(messageToEncrypt, key).toString();
                addText(encrypted, 'messageEncrypted');
                removeDisabled('messageEncrypted');
            }
            catch(error) {
                document.getElementById("errorSymEncrypt").innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Erreur de chiffrement symétrique : ' + error.message + '.</p>'
                return;
            }

        });

        // cancel
        var cancelSymEncrypt = document.getElementById('cancelSymEncrypt');
        if(cancelSymEncrypt){
            cancelSymEncrypt.addEventListener('click', function(e){
                e.preventDefault();
                var errorSymEncrypt = document.getElementById("errorSymEncrypt");
                errorSymEncrypt.innerHTML = '';

                const symEncryptKeyLocation = document.getElementById("symEncryptKeyLocation");
                    symEncryptKeyLocation.value = '';

                const message = document.getElementById("message");
                    message.value = '';

                const messageEncrypted = document.getElementById("messageEncrypted");
                    messageEncrypted.value = '';

                removeDisabled('buttonToEncrypt');
                setDisabled('messageEncrypted');
            })
        }
    }


    // ----------------------
    // TAB 3 - DECRYPT
    // ----------------------

    // button to decrypt
    var buttonToDecrypt = document.getElementById('buttonToDecrypt');
    if(buttonToDecrypt){
        buttonToDecrypt.addEventListener('click', function(e){
            e.preventDefault();

            var errorSymDecrypt = document.getElementById("errorSymDecrypt");
            errorSymDecrypt.innerHTML = '';

            var key = document.getElementById('symDecryptKeyLocation').value;  
            var messageToDecrypt = document.getElementById('messageToDecrypt').value;

            if('' == key) {                           
                document.getElementById("errorSymDecrypt").innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Merci d\'indiquer votre clé symétrique.</p>'
                return;
            }

            if('' == messageToDecrypt) {                           
                document.getElementById("errorSymDecrypt").innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Merci d\'indiquer votre contenu textuel à déchiffrer.</p>'
                return;
            }                   

            try {           
                // Decrypt
                var bytes  = CryptoJS.AES.decrypt(messageToDecrypt, key);
                var decrypted = bytes.toString(CryptoJS.enc.Utf8);
                addText(decrypted, 'messageDecrypted');
                removeDisabled('messageDecrypted');
            }
            catch(error) {
                document.getElementById("errorSymDecrypt").innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Erreur de déchiffrement symétrique : ' + error.message + '.</p>'
                return;
            }
        });

        // cancel
        var cancelSymDecrypt = document.getElementById('cancelSymDecrypt');
        if(cancelSymDecrypt){
            cancelSymDecrypt.addEventListener('click', function(e){
                e.preventDefault();
                var errorSymDecrypt = document.getElementById("errorSymDecrypt");
                errorSymDecrypt.innerHTML = '';
               
                const symDecryptKeyLocation = document.getElementById("symDecryptKeyLocation");
                symDecryptKeyLocation.value = '';

                const messageToDecrypt = document.getElementById("messageToDecrypt");
                messageToDecrypt.value = '';
                
                const messageDecrypted = document.getElementById("messageDecrypted");
                messageDecrypted.value = '';

                removeDisabled('buttonToDecrypt');
                setDisabled('messageDecrypted');
            })
        }
    }


    // --------------------------
    // TAB 4 - FIND BACK THE KEY
    // --------------------------

    function findBackKeyPromiseSuccess() {
        var elem = document.body;
        elem.style.cssText = '';
        var elemKey = document.getElementById('findBackKey');
        elemKey.removeAttribute('disabled', '');
    }

    function findBackKeyPromiseError() {
        document.getElementById("errorSymKeyBack").innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Un problème est survenu lors de la tentative de récupératon de la clé symétrique !</p>'
        return;
    }

    function findBackKeyFunction() {
        try {
            var findBackKeyPassphrase = document.getElementById('findBackKeyPassphrase').value;
            var findBackKeySalt = document.getElementById('findBackKeySalt').value;
            var findBackKey = createSymKey(findBackKeyPassphrase, findBackKeySalt);
            addText(findBackKey, 'findBackKey');
        }
        catch(error) {
            document.getElementById("errorSymKeyBack").innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Erreur de reconstitution de clé : ' + error.message + '.</p>'
            return;
        }
    }
    
    // button to find back the key
    var buttonFindBackKey = document.getElementById('buttonFindBackKey');
    if(buttonFindBackKey){
        buttonFindBackKey.addEventListener('click', function(e){

            var errorSymKeyBack = document.getElementById("errorSymKeyBack");
            errorSymKeyBack.innerHTML = '';

            var findBackKeyPassphrase = document.getElementById('findBackKeyPassphrase').value;
            var findBackKeySalt = document.getElementById('findBackKeySalt').value;
    
            if('' == findBackKeyPassphrase) {                           
                document.getElementById("errorSymKeyBack").innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Merci d\'indiquer votre mot de passe ou phrase secrète.</p>'
                return;
            }

            if('' == findBackKeySalt) {                           
                document.getElementById("errorSymKeyBack").innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Merci d\'indiquer le sel de votre clé.</p>'
                return;
            }      

            const findBackKeyPromise = new Promise((resolve, reject) => {
                var elem = document.body;
                elem.style.cssText = styleCover;

                setTimeout(() => {
                    resolve(findBackKeyFunction());
                }, 50);
            });
            findBackKeyPromise
            .then(findBackKeyPromiseSuccess, findBackKeyPromiseError);
        });
    }

    // ---------------------------------
    // OTP Section - One Time Pad (XOR)
    // ---------------------------------

    function convertToHex(strToConvertToHex) {       
        strToConvertToHex = utf8_to_b64(strToConvertToHex);

        var hex = '';
        for(var i = 0; i < strToConvertToHex.length; i++) {
            hex += '' + strToConvertToHex.charCodeAt(i).toString(16);
        }
        return hex;
    }


    // ------------
    // OTP - CRYPT
    // ------------
    var buttonOtpCrypt = document.getElementById('buttonOtpCrypt');
    if(buttonOtpCrypt){
        buttonOtpCrypt.addEventListener('click', function(e){
            e.preventDefault();
            var errorOtpCrypt = document.getElementById("errorOtpCrypt");
            errorOtpCrypt.innerHTML = '';

            var xorACrypt = document.querySelector('#xorACrypt').value;
            if('' == xorACrypt) {                           
                errorOtpCrypt.innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Merci d\'indiquer un masque jetable (OTP).</p>'
                return;
            }
            // clear text message
            var xorBCrypt = document.querySelector('#xorBCrypt').value;
            if('' == xorBCrypt) {                           
                errorOtpCrypt.innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Merci d\'indiquer un message en clair à chiffrer.</p>'
                return;
            }               

            if(xorACrypt.length != xorBCrypt.length) {                           
                errorOtpCrypt.innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Merci d\'indiquer un masque jetable aussi long que le message en clair à chiffrer.</p>'
                return;
            } 
            
            try {
                var xorACryptHex = xorACrypt;
                var xorBCryptHex = xorBCrypt;
                var xorCryptResult = common.xorHex(xorACryptHex, xorBCryptHex).toUpperCase();
                
                addText(xorCryptResult, "xorCryptResult");
                setDisabled('buttonOtpCrypt');
                removeDisabled('xorCryptResult');
            } 
            catch (e) {
                errorOtpCrypt.innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Chiffrement (OTP) : ' + e.message + '.</p>'
                return;
            }
        });

        // cancel
        var cancelOtpCrypt = document.getElementById('cancelOtpCrypt');
        if(cancelOtpCrypt){
            cancelOtpCrypt.addEventListener('click', function(e){
                e.preventDefault();
                var errorOtpCrypt = document.getElementById("errorOtpCrypt");
                errorOtpCrypt.innerHTML = '';

                const xorACrypt = document.getElementById("xorACrypt");
                    xorACrypt.value = '';

                const xorBCrypt = document.getElementById("xorBCrypt");
                    xorBCrypt.value = '';

                const xorCryptResult = document.getElementById("xorCryptResult");
                    xorCryptResult.value = '';

                removeDisabled('buttonOtpCrypt');
                setDisabled('xorCryptResult');
            })
        }
    }

    // A
    var charCounterXorA = document.getElementById("charCounterXorA");   
    var xorACryptTextarea = document.querySelector('#xorACrypt');
    const countCharactersXorA = () => {
        let numOfEnteredChars = xorACryptTextarea.value.length;
        charCounterXorA.textContent = numOfEnteredChars;
    };
    xorACryptTextarea.addEventListener("input", countCharactersXorA);

    // B
    var charCounterXorB = document.getElementById("charCounterXorB");
    var xorBCryptTextarea = document.querySelector('#xorBCrypt');
    const countCharactersXorB = () => {
        let numOfEnteredChars = xorBCryptTextarea.value.length;
        charCounterXorB.textContent = numOfEnteredChars;
    };
    xorBCryptTextarea.addEventListener("input", countCharactersXorB);

    // -------------
    // OTP - DECRYPT
    // -------------
    var buttonOtpDecrypt = document.getElementById('buttonOtpDecrypt');
    if(buttonOtpDecrypt){
        buttonOtpDecrypt.addEventListener('click', function(e){
            e.preventDefault();
            var errorOtpDecrypt = document.getElementById("errorOtpDecrypt");
            errorOtpDecrypt.innerHTML = '';

            var xorADecrypt = document.querySelector('#xorADecrypt').value;
            if('' == xorADecrypt) {                           
                errorOtpDecrypt.innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Merci d\'indiquer un masque jetable (OTP).</p>'
                return;
            }
            // clear text message
            var xorBDecrypt = document.querySelector('#xorBDecrypt').value;
            if('' == xorBDecrypt) {                           
                errorOtpDecrypt.innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Merci d\'indiquer un message à déchiffrer.</p>'
                return;
            }               

            if(xorADecrypt.length != xorBDecrypt.length) {                           
                errorOtpDecrypt.innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Merci d\'indiquer un masque jetable aussi long que le message en clair à chiffrer.</p>'
                return;
            } 
            
            try {
                var xorADecryptHex = xorADecrypt;
                var xorBDecryptHex = xorBDecrypt;
                var xorDecryptResult = common.xorHex(xorADecryptHex, xorBDecryptHex).toUpperCase();
                
                addText(xorDecryptResult, "xorDecryptResult");
                setDisabled('buttonOtpDecrypt');
                removeDisabled('xorDecryptResult');
            } 
            catch (e) {
                errorOtpDecrypt.innerHTML = '<p class="alert alert-danger"><i class="bi bi-exclamation-circle"></i> Déchiffrement (OTP) : ' + e.message + '.</p>'
                return;
            }
        });

        // cancel
        var cancelOtpDecrypt = document.getElementById('cancelOtpDecrypt');
        if(cancelOtpDecrypt){
            cancelOtpDecrypt.addEventListener('click', function(e){
                e.preventDefault();
                var errorOtpDecrypt = document.getElementById("errorOtpDecrypt");
                errorOtpDecrypt.innerHTML = '';

                const xorADecrypt = document.getElementById("xorADecrypt");
                    xorADecrypt.value = '';

                const xorBDecrypt = document.getElementById("xorBDecrypt");
                    xorBDecrypt.value = '';

                const xorDecryptResult = document.getElementById("xorDecryptResult");
                    xorDecryptResult.value = '';

                removeDisabled('buttonOtpDecrypt');
                setDisabled('xorDecryptResult');
            })
        }
    }

    // A
    var charCounterXorADecrypt = document.getElementById("charCounterXorADecrypt");   
    var xorADecryptTextarea = document.querySelector('#xorADecrypt');
    const countCharactersXorADecrypt = () => {
        let numOfEnteredChars = xorADecryptTextarea.value.length;
        charCounterXorADecrypt.textContent = numOfEnteredChars;
    };
    xorADecryptTextarea.addEventListener("input", countCharactersXorADecrypt);

    // B
    var charCounterXorBDecrypt = document.getElementById("charCounterXorBDecrypt");
    var xorBDecryptTextarea = document.querySelector('#xorBDecrypt');
    const countCharactersXorBDecrypt = () => {
        let numOfEnteredChars = xorBDecryptTextarea.value.length;
        charCounterXorBDecrypt.textContent = numOfEnteredChars;
    };
    xorBDecryptTextarea.addEventListener("input", countCharactersXorBDecrypt);

    
})();


